/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/

/**
 *  \file     neci_demo.cpp
 *  \brief    The program is a console application to demo using neci library interface.\n
 *
 *  2010-02-03	General Chiang\n
 *	new release\n
 */
#include <winsock2.h>
#include <stdio.h>
#include <conio.h>
#include "NECI.h"

#pragma comment(lib,"neci")
#pragma comment(lib,"ws2_32")

static char  USAGE[] =	
				"Usage:\r\n"\
				"\tneci_demo [action] [parameters]\r\n\r\n"\
				"neci_demo action\r\n"\
				"\t/search      Perform a broadcast search and report for local-linked \r\n"\
				"\t             devices found.\r\n"\
				"\t/locate      Start or stop to locate the device of specified IP.\r\n"\
				"\t/setip       Change the IP configuration of the device of specified IP.\r\n"\
				"\t/saverestart Save the configuration to flash and restart the device of\r\n"\
				"\t             specified IP.\r\n\r\n"\
				"neci_demo parameter\r\n"\
				"\t/locate	IP on|off\r\n"\
				"\t\tIP : IP address of the device.\r\n"\
				"\t\ton : Start to locate.\r\n"\
				"\t\toff : Stop to locate.\r\n"\
				"\t/setip IP new_IP\r\n"\
				"\t\tIP : IP address of the device.\r\n"\
				"\t\tnew_IP : new IP address of the device.\r\n"\
				"\t/saverestart IP \r\n"\
				"\t\tIP : IP address of the device.\r\n"\
				"Examples\r\n"\
				"\tneci_demo /search\r\n"\
				"\tneci_demo /locate 192.168.32.1.1 on\r\n"\
				"\tneci_demo /setip 192.168.32.1.1 192.168.1.2\r\n"\
				"\tneci_demo /saverestart 192.168.32.1.1\r\n";	/**< To show the program usage. */
#define ACTION_SEARCH	"/search"	/**< Perform a broadcast search and report for local-linked devices found. */
#define ACTION_LOCATE	"/locate"	/**< Start/Stop to locate the device of specified IP. */
#define ACTION_SETIP	"/setip"		/**< Change the IP configuration of the specified device. */
#define ACTION_SAVERESTART "/saverestart"	/**< Save the configuration to flash and restart the specified device. */
#define PARA_LOCATE_ON	"on"	/**< the 1st parameter of "locate on" action */
#define PARA_LOCATE_OFF	"off"	/**< the 1st parameter of "locate off" action */
#define NECI_SUCCESS	0	/**< Function succeeds. */
#define NECI_FAIL		-1	/**< Function fails. */
#define MAX_DEVICE_COUNTS	16	/**< Max. device counts to manage in this program. */
#define PASS_MASK	0x01	/**< the mask whether this device must login before using dsc_setXXX functions */
#define STREQ(a, b)	(strcmp((a), (b)) == 0)
#define DBG(x) printf x

/**< \struct _action_item
 *	action item information
 */
struct _action_item
{
	char	*action;		/**< action string\n */
	int		para_counts;		/**< parameter counts\n */
	int		(*fun)(char* argv[]);	/**< the callback function of the action\n */
};


/**< \struct _CONFIGINFO
 *	the management information of the device
 */
typedef struct _CONFIGINFO
{
    DS_INFO		ds_info;	/**< The device basic information such as application ID, hardware ID, IP address and MAC address.*/
	DS_HANDLE	dshdl;		/**< Device handle identifies returned from dsc_Attach. */
	PIFCONFIGINFO	p_ifconfig;	/**< Ifconfig of the device */
	PKERNELINFO		pkernel_info;	/**< The device kernel information */
} CFGINFO, *PCFGINFO;

/**< \struct _model_info
 *	model information
 */
struct _model_info
{
	int		hwid;		/**< hardware id\n */
	char	*name;		/**< model name\n */
};

static struct _model_info support_model[] =
{
	{MIINEPORTE2,	"MiiNePort E2"},
	{MIINEPORTE2_H,	"MiiNePort E2-H"},
	{MIINEPORTE3,	"MiiNePort E3"},
	{MIINEPORTE3_H,	"MiiNePort E3-H"},
	{NULL}
};

static int	SearchCount = 0;	/**< The counts of found device */
static CFGINFO	SearchInfo[MAX_DEVICE_COUNTS];

int action_fun_search(char* argv[]);
int CALLBACK EnumSearchProc(PDS_INFO dsinfo);
void show_search_result(void);
int action_fun_search(char* argv[]);
int action_fun_locate(char* argv[]);
int action_fun_setip(char* argv[]);
int action_fun_saverestart(char* argv[]);
static int check_login(CFGINFO *cfg_info);
static int init_CFGINFO(CFGINFO *cfg_info);
static void end_CFGINFO(CFGINFO *cfg_info);
static void detach_all_device();

static _action_item	neci_action[] =
{
	{ACTION_SEARCH,			0,	action_fun_search},
	{ACTION_LOCATE,			2,	action_fun_locate},
	{ACTION_SETIP,			2,	action_fun_setip},
	{ACTION_SAVERESTART,	1,	action_fun_saverestart},
	{NULL}
};

/**
 * \brief	The neci_demo program entry\n
 * \retval	 NECI_SUCCESS    The program excute successfully.\n
 * \retval	 NECI_FAIL    The program is excute fail.\n
 */
int main(int argc, char* argv[])
{
	int i = 0, ret;

	if (argc < 2)
	{
		printf("%s\r\n", USAGE);
		return NECI_FAIL;
	}

	while(neci_action[i].action)
	{
		if (STREQ(neci_action[i].action, argv[1]))
		{
			if (neci_action[i].para_counts > argc - 2)
			{
				printf("Usage:\r\n");
				printf("%s\r\n", USAGE);
				return NECI_FAIL;
			}
			else
			{
				ret = neci_action[i].fun(&argv[2]);
				if (ret == NECI_FAIL)
					printf("%s fails\r\n", &argv[1][1]);
				else
					printf("%s succeeds\r\n", &argv[1][1]);
				return ret;
			}
		}
		else
			i++;
	}
	return NECI_FAIL;
}
/**
 * \brief	The broadcast search routine.\n
 * <b>detail description</b>\n
 *	This function will use broadcast to search device.\n
 *				When search is finished, it will show the information of found device.</b>\n
 * \param	argv	The parameters.
 * \retval	NECI_SUCCESS	The function succeeds.\n
 * \retval	NECI_FAIL		The function fails.\n
 */
int action_fun_search(char* argv[])
{
	int ret;

	detach_all_device();
	/* broadcast search */
	if ((ret = dsc_EnumSearch(INADDR_BROADCAST, EnumSearchProc)) != DSC_OK)
	{
		DBG(("dsc_EnumSearch() fails, error code: %d\r\n", ret));
		ret = NECI_FAIL;
	}
	else
	{
		show_search_result();
		ret = NECI_SUCCESS;
	}
	detach_all_device();
	return ret;
}

/**
 * \brief	Try to locate on/off the specified device.\n
 * <b>detail description</b>\n
 *				Try to locate on/off the specified device. \n
 *				Starting to locate the specified device, \n
 *				it will response with blinking the READY light,\n
 *				backlight of the LCM On or continuous short beep.\n
 * \param	argv	The parameters.
 * \retval	NECI_SUCCESS	The function succeeds.\n
 * \retval	NECI_FAIL		The function fails.\n
 */
int action_fun_locate(char* argv[])
{
	int ret;
	u_long ip;

	if ((ip = inet_addr(argv[0])) == INADDR_NONE)
	{
		DBG(("Specified IP is invalid\r\n"));
		return NECI_FAIL;
	}

	/* search specific device */
	detach_all_device();
	if ((ret = dsc_EnumSearch(ip, EnumSearchProc)) != DSC_OK)
	{
		DBG(("dsc_EnumSearch() fails, error code: %d. \r\n", ret));
		return NECI_FAIL;
	}
	else
	{
		if (SearchCount != 1)
		{
			DBG(("Could not find the device\r\n"));
			return NECI_FAIL;
		}

		if (init_CFGINFO(&SearchInfo[0]) == NECI_FAIL)
		{
			DBG(("init_CFGINFO fails\r\n"));
			detach_all_device();
			return NECI_FAIL;
		}

		if (STREQ(argv[1], PARA_LOCATE_ON))
			ret = dsc_Locate(SearchInfo[0].dshdl, 1);
		else if (STREQ(argv[1], PARA_LOCATE_OFF))
			ret = dsc_Locate(SearchInfo[0].dshdl, 0);
		else
		{
			DBG(("The parameter [on/off] is error\r\n"));
			detach_all_device();
			return NECI_FAIL;
		}

		if (ret == DSC_OK)
			ret = NECI_SUCCESS;
		else
		{
			DBG(("dsc_Locate() fails, error code: %d.\r\n", ret));
			ret = NECI_FAIL;
		}

		detach_all_device();
		return ret;
	}
}

/**
 * \brief	Set the specified network interface IP address.\n
 * \param	argv	The parameters.
 * \retval	NECI_SUCCESS	The function succeeds.\n
 * \retval	NECI_FAIL		The function fails.\n
 */
int action_fun_setip(char* argv[])
{
	int ret;
	u_long ip;

	if ((ip = inet_addr(argv[0])) == INADDR_NONE)
	{
		DBG(("Specified IP is invalid\r\n"));
		return NECI_FAIL;
	}

	/* search specific device */
	detach_all_device();
	if ((ret = dsc_EnumSearch(ip, EnumSearchProc)) != DSC_OK)
	{
		DBG(("dsc_EnumSearch() failed, error code: %d.\r\n", ret));
		return NECI_FAIL;
	}
	else
	{
		if (SearchCount != 1)
		{
			DBG(("Could not find the device\r\n"));
			return NECI_FAIL;
		}

		if ((ip = inet_addr(argv[1])) == INADDR_NONE)
		{
			DBG(("New IP is invalid\r\n"));
			detach_all_device();
			return NECI_FAIL;
		}

		if (init_CFGINFO(&SearchInfo[0]) == NECI_FAIL)
		{
			DBG(("init_CFGINFO fails\r\n"));
			detach_all_device();
			return NECI_FAIL;
		}

		/* check login */
		if (check_login(&SearchInfo[0]) == NECI_FAIL)
		{
			DBG(("check_login fail\r\n"));
			detach_all_device();
			return NECI_FAIL;
		}

		/* set device ip */
		SearchInfo[0].p_ifconfig->ipaddr = ip;
		if ((ret = dsc_SetIP(SearchInfo[0].dshdl, SearchInfo[0].p_ifconfig->ifid, SearchInfo[0].p_ifconfig->ipaddr)) != DSC_OK)
		{
			DBG(("dsc_SetIP() fails, error code: %d.\r\n", ret));
			ret =  NECI_FAIL;
		}
		else
			ret = NECI_SUCCESS;
		detach_all_device();
		return ret;
	}
}

/**
 * \brief	Save the current settings and restart the device.\n
 * \param	argv	The parameters.
 * \retval	NECI_SUCCESS	The function succeeds.\n
 * \retval	NECI_FAIL		The function fails.\n
 */
int action_fun_saverestart(char* argv[])
{
	int ret;
	u_long ip;

	if ((ip = inet_addr(argv[0])) == INADDR_NONE)
	{
		DBG(("Specified IP is invalid\r\n"));
		return NECI_FAIL;
	}

	/* search specific device */
	detach_all_device();
	if ((ret = dsc_EnumSearch(ip, EnumSearchProc)) != DSC_OK)
	{
		printf("dsc_EnumSearch() fails, error code: %d \r\n", ret);
		return NECI_FAIL;
	}
	else
	{
		if (SearchCount != 1)
		{
			DBG(("Could not find the device\r\n"));
			return NECI_FAIL;
		}

		if (init_CFGINFO(&SearchInfo[0]) == NECI_FAIL)
		{
			DBG(("init_CFGINFO fails\r\n"));
			detach_all_device();
			return NECI_FAIL;
		}

		if (check_login(&SearchInfo[0]) == NECI_FAIL)
		{
			DBG(("check_login fail\r\n"));
			detach_all_device();
			return NECI_FAIL;
		}

		ret = dsc_SaveAndRestart(SearchInfo[0].dshdl);
		if (ret == DSC_OK)
			ret = NECI_SUCCESS;
		else
		{
			DBG(("dsc_SaveAndRestart() fails, error code: %d \r\n", ret));
			ret = NECI_FAIL;
		}
		detach_all_device();
		return ret;
	}
}
/**
 * \brief	An application-defined callback function used for the dsc_EnumSearch function.\n
 * \param	dsinfo	Pointer to a DS_INFO structure.\n
 * \retval	TRUE 	To continue enumeration.\n
 * \retval	FALSE	To stop enumeration.\n
 */
int CALLBACK EnumSearchProc(PDS_INFO dsinfo)
{
	if (dsinfo == NULL)
		return TRUE;

	SearchInfo[SearchCount].dshdl = -1;
	switch (dsinfo->hwid)
	{
	case MIINEPORTE2 :
	case MIINEPORTE2_H :
	case MIINEPORTE3 :
	case MIINEPORTE3_H :
		memcpy(&SearchInfo[SearchCount].ds_info, dsinfo, sizeof(DS_INFO));
		SearchCount++;
		break;
	default:
		break;
	}

	if (SearchCount > MAX_DEVICE_COUNTS)
		return FALSE;
	else
		return TRUE;
}
/**
 * \brief	Show the search result on the screen.\n
 */
void show_search_result(void)
{
	int i, j;
	struct in_addr	ip;

	printf("Search Result: ");
	if (SearchCount == 0)
	{
		printf("Not Found\r\n");
		return;
	}

	printf("\r\nNum\tModel\t\tIP Address\t\tMAC Address\r\n");
	for (i = 0; i < SearchCount; i++)
	{
		ip.S_un.S_addr = SearchInfo[i].ds_info.real_ip_addr;
		j = 0;
		while (support_model[j].hwid)
		{
			if (SearchInfo[i].ds_info.hwid == support_model[j].hwid)
			{
				printf("%d\t%s\t%s\t\t", i + 1, support_model[j].name, inet_ntoa(ip));
				break;
			}
			j++;
		}
		for (j = 0; j < 6; j++)
		{
			printf("%02X", SearchInfo[i].ds_info.mac[j]);
			if (j < 5)
				printf(":");
		}
		printf("\r\n");
	}
	printf("\r\n");
}
/**
 * \brief	Check if this device needs login before using dsc_SetXXX.\n
 * \param	cfg_info	Pointer to a CONFIGINFO structure.\n
 * \retval	NECI_SUCCESS	The function succeeds.\n
 * \retval	NECI_FAIL		The function fails.\n
 */
static int check_login(CFGINFO *cfg_info)
{
	char	password[30];
	int		i, ret;

	sprintf(password, "");
	if (cfg_info->pkernel_info->flag & PASS_MASK)
	{
		printf("Please input Password: ");
		i = 0;
		while (1)
		{
			password[i] = getch();
			if (password[i] != 0x0d)
			{
				printf("*");
				i++;
			}
			else
			{
				password[i] = NULL;
				break;
			}
		}
		printf("\r\n");
	}
	if ((ret = dsc_Login(cfg_info->dshdl, password)) != DSC_OK)
	{
		DBG(("dsc_Login() fails, error code: %d.\r\n",ret));
		return NECI_FAIL;
	}
	return NECI_SUCCESS;
}
/**
 * \brief	Inital the information of the found device.\n
 * \param	cfg_info	Pointer to a CONFIGINFO structure.\n
 * \retval	NECI_SUCCESS	The function succeeds.\n
 * \retval	NECI_FAIL		The function fails.\n
 */
static int init_CFGINFO(CFGINFO *cfg_info)
{
	int retcnt, ret;

	/* Attach the specified device and get the handle for retrieving the device detail information. */
	if ((ret = dsc_Attach(&cfg_info->ds_info, &cfg_info->dshdl)) != DSC_OK)
	{
		DBG(("dsc_Attach() fails, error code: %d. \r\n", ret));
		return NECI_FAIL;
	}
	cfg_info->p_ifconfig = (PIFCONFIGINFO)malloc(sizeof(IFCONFIGINFO));
	//cfg_info->p_ifconfig = new IFCONFIGINFO;
	/* Get the network interface information of the device */
	if ((ret = dsc_GetIfConfig(cfg_info->dshdl, 1, &retcnt, cfg_info->p_ifconfig)) != DSC_OK)
	{
		DBG(("dsc_GetIfConfig() fails, error code: %d.\r\n\r\n",ret));
		return NECI_FAIL;
	}
	cfg_info->pkernel_info = (PKERNELINFO)malloc(sizeof(KERNELINFO));
	//cfg_info->pkernel_info = new KERNELINFO;
	if ((ret = dsc_GetKernelInfo(cfg_info->dshdl, cfg_info->pkernel_info)) != DSC_OK)
	{
		DBG(("dsc_GetKernelInfo() fails, error code: %d.\r\n\r\n",ret));
		return NECI_FAIL;
	}
	return NECI_SUCCESS;
}

/**
 * \brief	detach & free CFGINFO of found device.\n
 * \param	cfg_info	Pointer to a CONFIGINFO structure.\n
 */
static void end_CFGINFO(CFGINFO *cfg_info)
{	
	dsc_Detach(cfg_info->dshdl);
	free(cfg_info->p_ifconfig);
	free(cfg_info->pkernel_info);
}

static void detach_all_device()
{
	int i;
	for (i = 0; i < SearchCount; i++)
	{
		if(SearchInfo[i].dshdl >= 0)				
			end_CFGINFO(&SearchInfo[i]);
	}
	SearchCount = 0;
}